local Tablet = Rock("Tablet-2.0", false, true)

RaidSaveFu = Rock:NewAddon("RaidSaveFu", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockConfig-1.0", "LibRockConsole-1.0", "LibFuBarPlugin-3.0")
local RaidSaveFu = RaidSaveFu
RaidSaveFu.title = "RaidSaveFu"
RaidSaveFu:SetDatabase("RaidSaveFuDB");

RaidSaveFu:SetDatabaseDefaults('profile', {
	debug = false,
	textwhenzero = false,
	shorttext = false,
	showid = true,
	showcooldown = true,
	showreset = true,
	dateformat = "%c"
} )
RaidSaveFu:SetFuBarOption('tooltipType', "Tablet-2.0")
RaidSaveFu:SetFuBarOption('iconPath', "Interface\\Addons\\FuBar_RaidSaveFu\\icon.tga")
RaidSaveFu:SetFuBarOption('hasNoColor', true)
RaidSaveFu:SetFuBarOption('cannotHideText', false)
--RaidSaveFu:SetFuBarOption('clickableTooltip', true)

function RaidSaveFu:OnInitialize()
	local optionsTable = {
		name = "FuBar_RaidSaveFu",
		desc = self.notes,
		handler = RaidSaveFu,
		type = 'group',
		args = {
			debug = {
				type = 'toggle',
				name = "Debug mode",
				desc = "Enable or disable extra diagnostic messages",
				get = "IsDebug",
				set = "ToggleDebug",
			},
			twentyFour = {
				type = 'toggle',
				name = "Hide text for no saved instances",
				desc = "Show or hide the FuBar text when the character has no saved instances",
				get = "IsTextWhenZero",
				set = "ToggleTextWhenZero",
			},
			showID = {
				type = 'toggle',
				name = "Show instance IDs",
				desc = "Show or hide the numeric instance IDs",
				get = "IsShowID",
				set = "ToggleShowID",
			},
			shortText = {
				type = 'toggle',
				name = "Short FuBar text",
				desc = "Only show the number of saved instances (no extra text)",
				get = "IsShortText",
				set = "ToggleShortText",
			},
			showCooldown = {
				type = 'toggle',
				name = "Show time remaining",
				desc = "Display the time the instance ID is still valid",
				get = "IsShowCooldown",
				set = "ToggleShowCooldown",
			},
			showReset = {
				type = 'toggle',
				name = "Show reset time",
				desc = "Display the time the instance ID expires",
				get = "IsShowReset",
				set = "ToggleShowReset",
			},
		}
	}
	RaidSaveFu:SetConfigTable(optionsTable)
	RaidSaveFu:SetConfigSlashCommand("/raidsavefu")
	RaidSaveFu.OnMenuRequest = optionsTable
	if self.db.profile.realm == nil then
		self.db.profile.realm = { }
	end
	if self.db.profile.realm[GetRealmName()] == nil then
		self.db.profile.realm[GetRealmName()] = { }
	end
	if self.db.profile.realm[GetRealmName()].char == nil then
		self.db.profile.realm[GetRealmName()].char = { }
	end
	if self.db.profile.realm[GetRealmName()].name == nil then
		self.db.profile.realm[GetRealmName()].name = GetRealmName()
	end
	if self.db.profile.realm[GetRealmName()].char[GetUnitName("player")] == nil then
		self.db.profile.realm[GetRealmName()].char[GetUnitName("player")] = { }	
	end
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	if player.numsaved == nil then
		player.numsaved = 0
	end
	if player.instance == nil then
		player.instance = { }
	end
	player.name = GetUnitName("player")
	player.class = strupper(UnitClass("player"))
end

function RaidSaveFu:OnEnable()
	self:AddEventListener("Blizzard", "UPDATE_INSTANCE_INFO", "Refresh")
	self:AddEventListener("Blizzard", "RAID_INSTANCE_WELCOME", "RequestRefresh")
	self:AddEventListener("Blizzard", "CHAT_MSG_SYSTEM", "CheckSystemMessage")
end

function RaidSaveFu:OnDisable()
	self:RemoveEventListener("Blizzard", "UPDATE_INSTANCE_INFO")
	self:RemoveEventListener("Blizzard", "RAID_INSTANCE_WELCOME")
	self:RemoveEventListener("Blizzard", "CHAT_MSG_SYSTEM")
end

function RaidSaveFu:OnUpdateFuBarText()
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	local fubartext
	if self:IsShortText() then
		fubartext = string.format("%d", player.numsaved)
	elseif player.numsaved == 1 then
		fubartext = string.format("%d Saved Instance", player.numsaved)
	else
		fubartext = string.format("%d Saved Instances", player.numsaved)
	end
	
	if player.numsaved == 0 and self:IsTextWhenZero() then
		self:SetFuBarText("");
	else
		self:SetFuBarText(fubartext)
	end
end

function RaidSaveFu:OnUpdateFuBarTooltip()
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	local numcolumns = 2
	if not self:IsShowCooldown() and not self:IsShowReset() then
		numcolumns = 1
	end
	if player.numsaved > 0 then
		local displaytable = { }
		local i
		for i = 1, player.numsaved do
			if player.instance[i].expires > time() then
				local displayrow = { }
				local namestring = player.instance[i].name
				if self:IsShowID() then
					namestring = namestring..GRAY_FONT_COLOR_CODE.." ["..player.instance[i].id.."]|r"
				end
				if numcolumns == 2 then
					local expirestring = ""
					if self:IsShowCooldown() then
						if player.instance[i].expires - time() <= 86400 then --less than a day
							expirestring = expirestring..RED_FONT_COLOR_CODE..SecondsToTime(tostring(player.instance[i].expires - time())).."|r"
						else -- more than a day
							expirestring = expirestring..GREEN_FONT_COLOR_CODE..SecondsToTime(tostring(player.instance[i].expires - time())).."|r"
						end
					end
					if self:IsShowCooldown() and self:IsShowReset() then
						expirestring = expirestring.." "
					end
					if self:IsShowReset() then
						expirestring = expirestring..date(dateformat, player.instance[i].expires)
						expirestring = strsub(expirestring, 1, strlen(expirestring) - 3)
					end
					displayrow = { name = namestring, expires = expirestring }
				else -- numcolumns = 1
					displayrow = { name = namestring }
				end
				table.insert(displaytable, displayrow)
			end
		end
		if # displaytable > 0 then
			if numcolumns == 2 then
				local cat = Tablet:AddCategory(
					'columns', numcolumns,
					'text', self:GetClassColorCode(player.class)..player.name.."|r",
					'justify', "LEFT",
					'justify2', "RIGHT"
				)
				cat:AddLine(
					'text', HIGHLIGHT_FONT_COLOR_CODE.."Instance|r",
					'text2', HIGHLIGHT_FONT_COLOR_CODE.."Resets (approx.)|r"
				)
				local i
				for i = 1, # displaytable do
					cat:AddLine(
						'text', displaytable[i].name,
						'text2', displaytable[i].expires
					)
				end
			else -- numcolumns == 1
				local cat = Tablet:AddCategory(
					'columns', numcolumns,
					'text', self:GetClassColorCode(player.class)..player.name.."|r",
					'justify', "LEFT"
				)
				local i
				for i = 1, # displaytable do
					cat:AddLine(
						'text', displaytable[i].name
					)
				end
			end
		elseif self:IsDebug() then
			self:Print("all raid data for "..self:GetClassColorCode(player.class)..player.name.."|r has expired")
		end
	elseif self:IsDebug() then
		self:Print("found no raid data for "..self:GetClassColorCode(player.class)..player.name.."|r")
	end
	local char = self.db.profile.realm[GetRealmName()].char
	local k, v
	for k, v in pairs(char) do
		if v.numsaved > 0 and v.name ~= player.name then
			if self:IsDebug() then
				self:Print("found raid data for "..self:GetClassColorCode(v.class)..v.name.."|r")
			end
			local displaytable = { }
			local i
			for i = 1, v.numsaved do
				if v.instance[i].expires > time() then
				local displayrow = { }
				local namestring = v.instance[i].name
				if self:IsShowID() then
					namestring = namestring..GRAY_FONT_COLOR_CODE.." ["..v.instance[i].id.."]|r"
				end
				if numcolumns == 2 then
					local expirestring = ""
					if self:IsShowCooldown() then
						if v.instance[i].expires - time() <= 86400 then --less than a day
							expirestring = expirestring..RED_FONT_COLOR_CODE..SecondsToTime(tostring(v.instance[i].expires - time())).."|r"
						else -- more than a day
							expirestring = expirestring..GREEN_FONT_COLOR_CODE..SecondsToTime(tostring(v.instance[i].expires - time())).."|r"
						end
					end
					if self:IsShowCooldown() and self:IsShowReset() then
						expirestring = expirestring.." "
					end
					if self:IsShowReset() then
						expirestring = expirestring..date(dateformat, v.instance[i].expires)
						expirestring = strsub(expirestring, 1, strlen(expirestring) - 3)
					end
					displayrow = { name = namestring, expires = expirestring }
				else -- numcolumns = 1
					displayrow = { name = namestring }
				end
				table.insert(displaytable, displayrow)
				end
			end
			if # displaytable > 0 then
				if self:IsDebug() then
					self:Print("displaying raid data for "..self:GetClassColorCode(v.class)..v.name.."|r")
				end
				if numcolumns == 2 then
					local cat = Tablet:AddCategory(
						'columns', numcolumns,
						'text', self:GetClassColorCode(v.class)..v.name.."|r",
						'justify', "LEFT",
						'justify2', "RIGHT"
					)
					cat:AddLine(
						'text', HIGHLIGHT_FONT_COLOR_CODE.."Instance|r",
						'text2', HIGHLIGHT_FONT_COLOR_CODE.."Resets (approx.)|r"
					)
					local i
					for i = 1, # displaytable do
						cat:AddLine(
							'text', displaytable[i].name,
							'text2', displaytable[i].expires
						)
					end
				else -- numcolumns == 1
					local cat = Tablet:AddCategory(
						'columns', numcolumns,
						'text', self:GetClassColorCode(v.class)..v.name.."|r",
						'justify', "LEFT"
					)
					local i
					for i = 1, # displaytable do
						cat:AddLine(
							'text', displaytable[i].name
						)
					end
				end
			elseif self:IsDebug() then
				self:Print("all raid data for "..self:GetClassColorCode(player.class)..player.name.."|r has expired")
			end
		end
	end
end

function RaidSaveFu:CheckSystemMessage()
	if tostring(arg1) == INSTANCE_SAVED then
		self:RequestRefresh()
	end
end

function RaidSaveFu:RequestRefresh()
	if self:IsDebug() then
		self:Print("requesting saved instance info")
	end
	RequestRaidInfo()
end

function RaidSaveFu:Refresh()
	if self:IsDebug() then
		self:Print("saved instance info received")
	end
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	player.numsaved = GetNumSavedInstances()
	player.instance = { }
	if player.numsaved == 0 then
	else
		local iname, iid, iexpires, temp
		for i = 1, player.numsaved do
			iname, iid, iexpires = GetSavedInstanceInfo(i)
			iexpires = iexpires + time()
			temp = { name = iname, id = iid, expires = iexpires }
			table.insert(player.instance, temp)
		end
	end
	self:UpdateFuBarPlugin()
end

function RaidSaveFu:GetDaysInMonth(year, month)
	local function is_leap_year(year)
		return year % 4 == 0 and (year % 100 ~= 0 or year % 400 == 0)
	end
	return month == 2 and is_leap_year(year) and 29 or ("\31\28\31\30\31\30\31\31\30\31\30\31"):byte(month)
end

function RaidSaveFu:GetClassColorCode(class)
	local r = string.format("%x", math.floor(RAID_CLASS_COLORS[class].r * 255))
	if strlen(r) == 1 then
		r = "0"..r
	end
	local g = string.format("%x", math.floor(RAID_CLASS_COLORS[class].g * 255))
	if strlen(g) == 1 then
		g = "0"..g
	end
	local b = string.format("%x", math.floor(RAID_CLASS_COLORS[class].b * 255))
	if strlen(b) == 1 then
		b = "0"..b
	end
	return "|cff"..r..g..b
end

-- ** functions for the UI configuration below **

function RaidSaveFu:IsTextWhenZero()
	return self.db.profile.textwhenzero
end

function RaidSaveFu:ToggleTextWhenZero()
	self.db.profile.textwhenzero = not self.db.profile.textwhenzero
	self:UpdateFuBarText()
	return self.db.profile.textwhenzero
end

function RaidSaveFu:IsShortText()
	return self.db.profile.shorttext
end

function RaidSaveFu:ToggleShortText()
	self.db.profile.shorttext = not self.db.profile.shorttext
	self:UpdateFuBarText()
	return self.db.profile.shorttext
end

function RaidSaveFu:GetDurationFormat()
	return self.db.profile.durationformat	
end

function RaidSaveFu:SelectDurationFormat(value)
	self.db.profile.durationformat = value
	self:UpdateFuBarTooltip()
	return self.db.profile.durationformat	
end

function RaidSaveFu:IsColorfulDuration()
	return self.db.profile.colorfulduration
end

function RaidSaveFu:ToggleColorfulDuration()
	self.db.profile.colorfulduration = not self.db.profile.colorfulduration
	self:UpdateFuBarTooltip()
	return self.db.profile.colorfulduration
end

function RaidSaveFu:IsShowID()
	return self.db.profile.showid
end

function RaidSaveFu:ToggleShowID()
	self.db.profile.showid = not self.db.profile.showid
	self:UpdateFuBarTooltip()
	return self.db.profile.showid
end

function RaidSaveFu:IsDebug()
	return self.db.profile.debug
end

function RaidSaveFu:ToggleDebug()
	self.db.profile.debug = not self.db.profile.debug
	self:UpdateFuBarTooltip()
	return self.db.profile.debug
end

function RaidSaveFu:IsShowCooldown()
	return self.db.profile.showcooldown
end

function RaidSaveFu:ToggleShowCooldown()
	self.db.profile.showcooldown = not self.db.profile.showcooldown
	self:UpdateFuBarTooltip()
	return self.db.profile.showcooldown
end

function RaidSaveFu:IsShowReset()
	return self.db.profile.showreset
end

function RaidSaveFu:ToggleShowReset()
	self.db.profile.showreset = not self.db.profile.showreset
	self:UpdateFuBarTooltip()
	return self.db.profile.showreset
end
